/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "new.h"

// This is the default constructor for the new project dialog. It is meant
// to initialize all of the objects and set their properties. It then creates
// the dialog and displays it to the screen
NewProjectDialog::NewProjectDialog (TemplateList& list_of_templates)
{
  dialog = this;
  tlist = list_of_templates;
  initialize_objects();
  
  dialog->get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
  dialog->set_default_icon_from_file(string(INSTALL_PREFIX) + "/share/pixmaps/openldev/openldev24.png");
  populate_combobox();
  
  combobox->set_size_request(200,30);
  name->set_size_request(200,25);
  version->set_size_request(200,25);
  homepage->set_size_request(200,25);
  email->set_size_request(200,25);
  author->set_size_request(200,25);
  
  // Set widgets to the Fixed box
  hbox3->set_border_width(10);
  hbox3->pack_start(*label, Gtk::PACK_SHRINK, 10);
  hbox3->pack_start(*combobox, Gtk::PACK_EXPAND_WIDGET, 10);
  
  // Create the labels for the file/project properties
  location->set_text(Glib::ustring(Glib::get_home_dir() + "/openldev/"));
  label1->set_alignment(1.0,0.5);
  label2->set_alignment(1.0,0.5);
  label3->set_alignment(1.0,0.5);
  label4->set_alignment(1.0,0.5);
  label5->set_alignment(1.0,0.5);
  label6->set_alignment(1.0,0.5);
  
  // Pack the browse & location in HBox2
  hbox2->set_spacing(5);
  hbox2->pack_start(*location);
  hbox2->pack_start(*browse, Gtk::PACK_SHRINK);

  // Add labels & entries to the H/VBox "table"
  vbox2->pack_start(*label5);
  vbox2->pack_start(*label6);
  vbox2->pack_start(*label4);
  vbox2->pack_start(*label3);
  vbox2->pack_start(*label2);
  vbox2->pack_start(*label1);
  vbox3->pack_start(*name);
  vbox3->pack_start(*hbox2);
  vbox3->pack_start(*author);
  vbox3->pack_start(*email);
  vbox3->pack_start(*homepage);
  vbox3->pack_start(*version);
  vbox2->set_size_request(-1, 275);
  vbox3->set_size_request(165, 275);
  hbox4->pack_start(*vbox2, Gtk::PACK_SHRINK, 10);
  hbox4->pack_start(*vbox3, Gtk::PACK_EXPAND_WIDGET, 10);
  
  // Create & pack the buttons
  cancel->set_size_request(85, 30);
  apply->set_size_request(85, 30);
  hbox->pack_end(*apply, Gtk::PACK_SHRINK, 6);
  hbox->pack_end(*cancel, Gtk::PACK_SHRINK, 6);
  
  // Add widgets to the overall vbox
  vbox->set_spacing(5);
  vbox->pack_start(*hbox3, Gtk::PACK_SHRINK);
  vbox->pack_start(*hseparator, Gtk::PACK_SHRINK);
  vbox->pack_start(*hbox4);
  vbox->pack_start(*hseparator2, Gtk::PACK_SHRINK, 3);
  vbox->pack_start(*hbox, Gtk::PACK_SHRINK);
  
  // Set the dialog properties
  dialog->get_vbox()->set_homogeneous(false);
  dialog->get_vbox()->set_spacing(0);
  dialog->get_vbox()->pack_start(*vbox);
  dialog->set_title("New Project");
  dialog->set_modal(true);
  dialog->property_window_position().set_value(Gtk::WIN_POS_NONE);
  dialog->set_resizable(false);
  dialog->property_destroy_with_parent().set_value(false);
  dialog->set_has_separator(false);

  dialog->show_all_children();
  dialog->show();
  
  apply->signal_clicked().connect(sigc::mem_fun(*this, &NewProjectDialog::on_apply));
  cancel->signal_clicked().connect(sigc::mem_fun(*this, &NewProjectDialog::on_cancel));
  browse->signal_clicked().connect(sigc::mem_fun(*this, &NewProjectDialog::on_browse));
}

// The dialog and its widgets are initialized in this function.
void NewProjectDialog::initialize_objects()
{
  cancel = new Gtk::Button(Gtk::StockID("gtk-cancel"));
  apply = new Gtk::Button(Gtk::StockID("gtk-ok"));
  browse = new Gtk::Button("...");
  hseparator = new Gtk::HSeparator();
  hseparator2 = new Gtk::HSeparator();
  combobox = new Gtk::Combo();
  
  vbox = new Gtk::VBox(false, 0);
  vbox2 = new Gtk::VBox(false, 0);
  vbox3 = new Gtk::VBox(false, 0);
  hbox = new Gtk::HBox(false, 0);
  hbox2 = new Gtk::HBox(false, 0);
  hbox3 = new Gtk::HBox(false, 0);
  hbox4 = new Gtk::HBox(false, 0);
  
  version = new Gtk::Entry();
  homepage = new Gtk::Entry();
  location = new Gtk::Entry();
  email = new Gtk::Entry();
  author = new Gtk::Entry();
  name = new Gtk::Entry();
  
  label = new Gtk::Label("Create New:");
  label1 = new Gtk::Label("Version:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  label2 = new Gtk::Label("Homepage:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  label3 = new Gtk::Label("E-mail:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  label4 = new Gtk::Label("Author:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  label5 = new Gtk::Label("Name:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  label6 = new Gtk::Label("Directory:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
}

// This destructor goes through all of the initialized widgets
// and deletes them to free up memory on close of the dialog
NewProjectDialog::~NewProjectDialog()
{
  delete cancel;
  delete apply;
  delete browse;
  delete label;
  delete label1;
  delete label2;
  delete label3;
  delete label4;
  delete label5;
  delete label6;
  delete vbox;
  delete hbox;
  delete hbox2;
  delete name;
  delete author;
  delete email;
  delete homepage;
  delete version;
  delete location;
  delete hbox3;
  delete combobox;
  delete hseparator;
  delete hseparator2;
}

// When cancel is pressed, this function is called to emit a cancel signal
// and hide the dialog
void NewProjectDialog::on_cancel(void)
{
  dialog->response(Gtk::RESPONSE_CANCEL);
  dialog->hide();
}

// When the browse button is pressed, the user is presented with a 
// FileChooserDialog where they can choose where to create the project.
void NewProjectDialog::on_browse(void)
{
  Gtk::FileChooserDialog dialog("Choose Directory", Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
  dialog.set_transient_for(*this);
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  
  int result = dialog.run();
  switch (result)
  {
    // If OK is pressed
    case (Gtk::RESPONSE_OK):
    {
      location->set_text(Glib::ustring(dialog.get_filename() + "/"));
      break;
    }
    default:
      break;
  }
}

// Check the dialog for the following errors: If the project name is empty or
// contains non-alphanumeric characters, if the version is empty, if the project
// locatino is empty
Glib::ustring NewProjectDialog::check_for_errors()
{
  Glib::ustring ntxt = name->get_text(), vtxt = version->get_text(),
                ltxt = location->get_text();
  
  // Check for error when creating a project or file
  if (ntxt.find(" ",0) != Glib::ustring::npos ||
      ntxt.find("_",0) != Glib::ustring::npos ||
      ntxt.empty())
    return "The project name must\nbe alphanumeric!";
  else if (vtxt.empty())
    return "You must specify a\nversion number!";
  else if (ltxt.empty())
    return "You must specify a\nproject location!";
  else
    return "NONE";
}

// Create the directory the project is to be created in
void NewProjectDialog::create_location()
{
  mkdir(location->get_text().c_str(), 0777);
}

// Create a new project file. This function returns text that is to be added
// to the project file and returns by reference the location the project file
// is to be created.
Glib::ustring NewProjectDialog::create_odp_file_text(Glib::ustring &dir)
{
  Gtk::Entry *choice = combobox->get_entry();
  Glib::ustring odp_file_text, choice_text = choice->get_text();
  
  // Create the text for the file
  odp_file_text = "<name>" + name->get_text() + "*\n<author>" + author->get_text()
    + "*\n<location>" + location->get_text() + name->get_text()
    + "/*\n<email>" + email->get_text() + "*\n<url>" + homepage->get_text() 
    + "*\n<version>" + version->get_text();
  
  // Add the files that need to be added for the template
  for (unsigned int i=0; i < tlist.title.size(); i++)
    if (tlist.title.at(i) == choice_text)
    {
      Glib::ustring fname = tlist.defaultfiles.at(i);
      odp_file_text += string("*\n<template>") + tlist.file.at(i) 
                    + "*\n<lastfile>" + fname.substr(0, fname.find("/", 0))
                    + "*\n<sources>" + tlist.defaultfiles.at(i) 
                    + "*\n<headers>*\n<otherfiles>*\n<execname>";
    }
    
  dir = location->get_text() + name->get_text() + "/" + name->get_text() + ".openldev";
  
  return odp_file_text;
}

// Run the commands to copy the template to the project directory, extract the
// tar template file and remove that tar file
void NewProjectDialog::run_copy_extract_rm (Glib::ustring copy,
                       Glib::ustring extract, Glib::ustring rm)
{
  // Make project directory & copy files over
  FILE *command;
  Glib::ustring project_path = location->get_text() + name->get_text() + "/";
  mkdir(project_path.c_str(), 0777);
  command = popen(copy.c_str(), "r");
  pclose(command);
    
  // Extract & delete file
  chdir(project_path.c_str());
  command = popen(extract.c_str(), "r");
  pclose(command);
  command = popen(rm.c_str(), "r");
  pclose(command);
}

// Find the instances of TITLE and VERSION and replace them with the
// title of the project and the version specified in the configure.in file
void NewProjectDialog::edit_configure_file (Glib::ustring configure_in)
{
  // Edit configure.in file
  Glib::ustring text = read_file(configure_in);
  int loc = text.find("TITLE",0);
  text.erase(loc, 5);
  text.insert(loc, name->get_text());
  loc = text.find("VERSION",0);
  text.erase(loc, 7);
  text.insert(loc, version->get_text());
  write_file(configure_in, text);
}

// Find all instances of TITLE in the Makefile.am files. Replace them with
// the title of the project specified by the users
void NewProjectDialog::edit_makefiles (Glib::ustring makefile_am,
                                       Glib::ustring makefile_am2)
{
  // Edit src/Makefile.am file
  Glib::ustring text = read_file(makefile_am);
  int loc = text.find("TITLE",0);
  while (loc != Glib::ustring::npos)
  {
    text.erase(loc,5);
    text.insert(loc,name->get_text());
    loc = text.find("TITLE",0);
  }
  write_file(makefile_am, text);
    
  // Edit Makefile.am file
  text = read_file(makefile_am2);
  loc = text.find("TITLE",0);
  while (loc != Glib::ustring::npos)
  {
    text.erase(loc,5);
    text.insert(loc,name->get_text());
    loc = text.find("TITLE",0);
  }
  write_file(makefile_am2, text);
}

// When the apply button is pressed, it is checked that the project name only
// contains letters and numbers and that a version and location is specified.
// The project is then created from a template and the Makefiles and configure
// file are edited. The OpenLDev project file is also created.
void NewProjectDialog::on_apply()
{
  Glib::ustring errortext = check_for_errors();
  
  // Create new project
  if (errortext == "NONE")
  {
    Glib::ustring odp_file_text, odp_file_location, project_path, copy, extract,
                  rm, configure_in, makefile_am, makefile_am2, text;
    // Add a '/' to the end of the location if it does not exist
    if (location->get_text().at(location->get_text().length()-1) != '/')
    {
      Glib::ustring str = location->get_text();
      str += "/";
      location->set_text(str);
    }

    // Create the project location, openldev file location and openldev file text
    create_location();
    odp_file_text = create_odp_file_text(odp_file_location);    
    
    // Create directory path
    project_path += location->get_text() + name->get_text() + "/";
    configure_in = project_path + "configure.in";
    makefile_am = project_path + "src/Makefile.am";
    makefile_am2 = project_path + "Makefile.am";
    
    // Create copy command, configure.in path, & makefile.am path
    Gtk::Entry *choice = combobox->get_entry();
    Glib::ustring choice_text = choice->get_text();
    
    for (unsigned int i=0; i < tlist.title.size(); i++)
    {
      if (tlist.title.at(i) == choice_text)
      {
        copy = "cp -rf " + string(INSTALL_PREFIX) + "/share/openldev/templates/" 
               + tlist.file.at(i) + ".tar " + project_path;
        extract = "tar -xf " + tlist.file.at(i) + ".tar";
        rm = "rm -f " + tlist.file.at(i) +  ".tar";
      }
    }
    
    // Create the project from the template & create the project file
    run_copy_extract_rm (copy, extract, rm);
    write_file(odp_file_location, odp_file_text);
    file_location = odp_file_location;
    
    // Edit the configure.in file and Makefile.am files
    edit_configure_file(configure_in);
    edit_makefiles(makefile_am, makefile_am2);
     
    dialog->response(Gtk::RESPONSE_ACCEPT);
    dialog->hide();
  }
  // Throw an error
  else
  {
    error(errortext);
  }
}

// This adds the different types of projects to the combobox
void NewProjectDialog::populate_combobox()
{ combobox->set_popdown_strings(tlist.title);
}

// This allows the main openldev class access to the filename of the project.
Glib::ustring NewProjectDialog::get_filename() 
{ return file_location; 
}
